// python wrapper for vtkArrayIterator object
//
#define VTK_WRAPPING_CXX
#define VTK_STREAMS_FWD_ONLY
#include "vtkPython.h"
#undef _XOPEN_SOURCE /* Conflicts with standards.h.  */
#undef _THREAD_SAFE /* Conflicts with pthread.h.  */
#include "vtkPythonUtil.h"
#include <vtksys/ios/sstream>
#include "vtkArrayIterator.h"
#if defined(WIN32)
extern "C" { __declspec( dllexport ) PyObject *PyVTKClass_vtkArrayIteratorNew(char *); }
#else
extern "C" { PyObject *PyVTKClass_vtkArrayIteratorNew(char *); }
#endif

extern "C" { PyObject *PyVTKClass_vtkObjectNew(char *); }

static PyObject *PyvtkArrayIterator_GetClassName(PyObject *self, PyObject *args)
{
  /* handle an occurrence */
  {
  vtkArrayIterator *op;

  const char    *temp20;
  op = (vtkArrayIterator *)PyArg_VTKParseTuple(self, args, (char*)"");
  if (op)
    {
    if (PyVTKClass_Check(self))
      {
      temp20 = op->vtkArrayIterator::GetClassName();
      }
    else
      {
      temp20 = op->GetClassName();
      }
    if (temp20 == NULL) {
      Py_INCREF(Py_None);
      return Py_None;
      }
    else {
      return PyString_FromString(temp20);
      }
    }
  }
  return NULL;
}


static PyObject *PyvtkArrayIterator_IsA(PyObject *self, PyObject *args)
{
  /* handle an occurrence */
  {
  vtkArrayIterator *op;

  char    *temp0;
  int      temp20;
  op = (vtkArrayIterator *)PyArg_VTKParseTuple(self, args, (char*)"z", &temp0);
  if (op)
    {
    if (PyVTKClass_Check(self))
      {
      temp20 = op->vtkArrayIterator::IsA(temp0);
      }
    else
      {
      temp20 = op->IsA(temp0);
      }
    return PyInt_FromLong(temp20);
    }
  }
  return NULL;
}


static PyObject *PyvtkArrayIterator_NewInstance(PyObject *self, PyObject *args)
{
  /* handle an occurrence */
  {
  vtkArrayIterator *op;

  vtkArrayIterator  *temp20;
  op = (vtkArrayIterator *)PyArg_VTKParseTuple(self, args, (char*)"");
  if (op)
    {
    if (PyVTKClass_Check(self))
      {
      temp20 = op->vtkArrayIterator::NewInstance();
      }
    else
      {
      temp20 = op->NewInstance();
      }
    return vtkPythonGetObjectFromPointer((vtkObjectBase *)temp20);
    }
  }
  return NULL;
}


static PyObject *PyvtkArrayIterator_Initialize(PyObject *self, PyObject *args)
{
  /* handle an occurrence */
  {
  vtkArrayIterator *op;

  vtkAbstractArray  *temp0;
  PyObject *tempH0;
  op = (vtkArrayIterator *)PyArg_VTKParseTuple(self, args, (char*)"O", &tempH0);
  if (op)
    {
    temp0 = (vtkAbstractArray *)vtkPythonGetPointerFromObject(tempH0,(char*)"vtkAbstractArray");
    if (!temp0 && tempH0 != Py_None) goto break3;
    if (PyVTKClass_Check(self))
      {
      PyErr_SetString(PyExc_TypeError,"pure virtual method call");
      return NULL;
    }
    else
      {
      op->Initialize(temp0);
      }
    Py_INCREF(Py_None);
    return Py_None;
    }
  }
 break3:
  return NULL;
}


static PyObject *PyvtkArrayIterator_GetDataType(PyObject *self, PyObject *args)
{
  /* handle an occurrence */
  {
  vtkArrayIterator *op;

  int      temp20;
  op = (vtkArrayIterator *)PyArg_VTKParseTuple(self, args, (char*)"");
  if (op)
    {
    if (PyVTKClass_Check(self))
      {
      PyErr_SetString(PyExc_TypeError,"pure virtual method call");
      return NULL;
    }
    else
      {
      temp20 = op->GetDataType();
      }
    return PyInt_FromLong(temp20);
    }
  }
  return NULL;
}

static PyMethodDef PyvtkArrayIteratorMethods[] = {
  {(char*)"GetClassName",                (PyCFunction)PyvtkArrayIterator_GetClassName, 1,
   (char*)"V.GetClassName() -> string\nC++: const char *GetClassName ();\n\n"},
  {(char*)"IsA",                (PyCFunction)PyvtkArrayIterator_IsA, 1,
   (char*)"V.IsA(string) -> int\nC++: int IsA (const char *name);\n\n"},
  {(char*)"NewInstance",                (PyCFunction)PyvtkArrayIterator_NewInstance, 1,
   (char*)"V.NewInstance() -> vtkArrayIterator\nC++: vtkArrayIterator *NewInstance ();\n\n"},
  {(char*)"Initialize",                (PyCFunction)PyvtkArrayIterator_Initialize, 1,
   (char*)"V.Initialize(vtkAbstractArray)\nC++: virtual void Initialize (vtkAbstractArray *array) = 0;\n\n Set the array this iterator will iterate over.\n After Initialize() has been called, the iterator is valid\n so long as the Array has not been modified \n (except using the iterator itself).\n If the array is modified, the iterator must be re-intialized.\n"},
  {(char*)"GetDataType",                (PyCFunction)PyvtkArrayIterator_GetDataType, 1,
   (char*)"V.GetDataType() -> int\nC++: virtual int GetDataType () = 0;\n\n"},
  {NULL,                       NULL, 0, NULL}
};

static const char *vtkArrayIteratorDoc[] = {
  (char*)"vtkArrayIterator - Abstract superclass to iterate over elements in \n\n",
  "Super Class:\n\n vtkObject\n\n",
  (char*)" vtkArrayIterator is used to iterate over elements in any vtkAbstractArray\n subclass.\n The vtkArrayIteratorTemplateMacro is used to centralize the set of types\n supported by Execute methods.  It also avoids duplication of long\n switch statement case lists.\n Note that in this macro VTK_TT is defined to be the type of the iterator\n for the given type of array. One must include the \n vtkArrayIterator",
  (char*)"Includes.h header file to provide for extending of this macro\n by addition of new iterators.\n\n Example usage:\n \\code\n vtkArrayIter* iter = array->NewIterator();\n switch(array->GetDataType())\n   {\n   vtkArrayIteratorTemplateMacro(myFunc(static_cast<VTK_TT*>(iter), arg2));\n   }\n iter->Delete();\n \\endcode\n\n",
  NULL
};

PyObject *PyVTKClass_vtkArrayIteratorNew(char *modulename)
{
  return PyVTKClass_New(NULL,
                        PyvtkArrayIteratorMethods,
                        (char*)"vtkArrayIterator",modulename,
                        (char**)vtkArrayIteratorDoc,
                        PyVTKClass_vtkObjectNew(modulename));
}

